"use strict";
const node_html_parser_1 = require("node-html-parser");
const fencBlockName = "named-fence-block";
const fenceFileName = "named-fence-filename";
const defaultStyleOptions = {
    mincbBlock: "position: relative; padding-top: 2em;",
    mincbName: "position: absolute; top: 0; left: 0; padding: 0 4px; " +
        "font-weight: bold; color: #000000; background: #c0c0c0; opacity: .6;"
};
const namedCodeBlocks = (md, options) => {
    var _a, _b;
    const isEnableInlineCss = (_a = options === null || options === void 0 ? void 0 : options.isEnableInlineCss) !== null && _a !== void 0 ? _a : false;
    const defaultRender = (_b = md.renderer.rules) === null || _b === void 0 ? void 0 : _b.fence;
    if (!defaultRender) {
        throw new Error("defaultRender is undefined");
    }
    md.renderer.rules.fence = (tokens, idx, options, env, self) => {
        const token = tokens[idx];
        const orgInfo = token.info;
        const info = token.info ? String(token.info).trim() : "";
        let parsedFenceInfo;
        if (info) {
            parsedFenceInfo = parseInfo(info);
            updateTokenInfo(token, parsedFenceInfo);
        }
        else {
            parsedFenceInfo = { langName: "", fileName: "", langAttrs: "" };
        }
        const rootElement = (0, node_html_parser_1.parse)(defaultRender(tokens, idx, options, env, self));
        token.info = orgInfo;
        updateElement(rootElement, parsedFenceInfo, isEnableInlineCss);
        return rootElement.toString();
    };
};
function updateElement(element, parsedFenceInfo, isEnableInlineCss) {
    if (parsedFenceInfo.fileName && parsedFenceInfo.langName) {
        addNamedFenceBlockAttr(element, isEnableInlineCss);
        addNamedFenceFilenameAtrr(element, parsedFenceInfo.fileName, isEnableInlineCss);
    }
}
function addNamedFenceBlockAttr(element, isEnableInlineCss) {
    if (element.firstChild instanceof node_html_parser_1.HTMLElement) {
        const existClass = element.firstChild.getAttribute("class");
        if (element.firstChild.getAttribute("class")) {
            element.firstChild.setAttribute("class", `${existClass} ${fencBlockName}`);
        }
        else {
            element.firstChild.setAttribute("class", fencBlockName);
        }
        if (isEnableInlineCss) {
            element.firstChild.setAttribute("style", defaultStyleOptions.mincbBlock);
        }
    }
}
function addNamedFenceFilenameAtrr(element, fileName, isEnableInlineCss) {
    const node = (0, node_html_parser_1.parse)(`<div class="${fenceFileName}">${fileName}</div>`);
    const firstChild = node.firstChild;
    if (firstChild instanceof node_html_parser_1.HTMLElement && isEnableInlineCss) {
        firstChild.setAttribute("style", defaultStyleOptions.mincbName);
    }
    if (element.firstChild instanceof node_html_parser_1.HTMLElement) {
        element.firstChild.appendChild(node);
    }
}
function parseInfo(info) {
    // https://regex101.com/r/PacPRb/4
    const data = { langName: "", fileName: "", langAttrs: "" };
    const arr = info.split(/(\s+)/g);
    const match = arr[0].match(/^([^:\n]+)?(:([^:\n]*))?([^:\n]*)?$/);
    const langAttrs = arr.slice(2).join("");
    if (match) {
        data.langName = match[1] || "";
        data.fileName = match[3] || "";
        data.langAttrs = langAttrs;
        return data;
    }
    return data;
}
function updateTokenInfo(token, parsedFenceInfo) {
    if (parsedFenceInfo.langName) {
        token.info = parsedFenceInfo.langName + " " + parsedFenceInfo.langAttrs;
    }
    else {
        token.info = "";
    }
}
module.exports = namedCodeBlocks;
